==> localstack-ext-1.3.2.dev20230209175603.tar.gz: localstack-ext-1.3.2.dev20230209175603/localstack_ext/bootstrap/pods/server/states.py <==
_A='global'
import logging,os
from typing import Dict,Optional,Set
from localstack import config
from localstack.aws.accounts import get_aws_account_id
from localstack.services.stores import AccountRegionBundle
from localstack.services.visitors import ReflectionStateLocator,ServiceBackend,ServiceBackendCollectorVisitor
from localstack.utils.aws.aws_stack import get_valid_regions_for_service
from localstack.utils.files import load_file
from localstack.utils.objects import singleton_factory
from localstack.utils.strings import to_bytes
from moto.core import BackendDict
from moto.s3.models import S3Backend
from plugin import PluginManager
from localstack_ext.bootstrap.pods.server.plugins import StateLifecyclePlugin
from localstack_ext.bootstrap.pods.server.visitors import PodStateLoaderVisitor
from localstack_ext.bootstrap.pods.service_state.service_state import ServiceState
from localstack_ext.bootstrap.pods.service_state.service_state_types import AccountRegion,BackendState,ServiceKey
from localstack_ext.constants import STORE_STATE_FILE
LOG=logging.getLogger(__name__)
PLUGIN_NAMESPACE='localstack.state.lifecycle'
class PersistencePluginManager(PluginManager[StateLifecyclePlugin]):
	def __init__(A):super().__init__(PLUGIN_NAMESPACE)
	@staticmethod
	@singleton_factory
	def get():return PersistencePluginManager()
	def get_plugin(B,service):
		A=service
		try:return B.load(A)
		except ValueError as C:LOG.debug('Failed to load persistence plugin for "%s": "%s',A,C)
class ServiceStateManager:
	def __init__(A,service):A.service=service;A.manager=PersistencePluginManager.get();A.manager.load_all()
	def get_service_state(A):
		D=ServiceBackendCollectorVisitor();G=ReflectionStateLocator(service=A.service);G.accept(visitor=D);E=D.collect();B=ServiceState()
		for F in E.keys():B.put_service_state(_service_state_from_backend(backend=E[F],api=A.service,memory_management=F))
		C=A.manager.get_plugin(A.service)
		if C and C.has_assets():H=C.retrieve_assets();B.put_assets(A.service,H)
		return B
	def inject_service_state(A,pod_state,pod_tmp_dir=None):
		from localstack.services.plugins import SERVICE_PLUGINS as E;F=PodStateLoaderVisitor(pod_state=pod_state,service=A.service);G=ReflectionStateLocator(service=A.service);G.accept(visitor=F);B=A.manager.get_plugin(A.service)
		if B:B.inject_assets(pod_asset_directory=pod_tmp_dir)
		C=E.get_service_container(A.service)
		if not C:LOG.debug('No service container for %s',A.service);return
		D=C.service.lifecycle_hook
		if D:
			try:D.on_after_inject()
			except Exception as H:LOG.debug('An error occurred while executing the after inject hook for %s: %s',A.service,H)
	def active_service_regions(B):
		A=B._active_service_regions()
		if B.service=='stepfunctions':
			C=os.path.join(config.dirs.data,B.service,'backend_state')
			if os.path.exists(C):
				try:D=load_file(C,mode='rb');A=[B for B in A if to_bytes(B.region)in D]
				except Exception:pass
			return set(A)
		return A
	def _active_service_regions(D):
		def B(reg):return AccountRegion(account_id=get_aws_account_id(),region=reg)
		def G(account_id):
			A=account_id
			if A=='123456789012':return get_aws_account_id()
			return A
		E=ServiceBackendCollectorVisitor();H=ReflectionStateLocator(service=D.service);H.accept(visitor=E);F=E.collect()
		if not F:I=get_valid_regions_for_service(D.service);return set([B(A)for A in I])
		C=set()
		for (M,A) in F.items():
			if isinstance(A,(BackendDict,AccountRegionBundle)):
				for (J,K) in A.items():
					for L in K.keys():C.add(AccountRegion(account_id=G(J),region=L))
			elif isinstance(A,dict):C.update(set([B(C)for C in A.keys()]))
			else:return{B(_A)}
		return C
def _service_state_from_account_region_bundle(service_backend,api):
	A=ServiceState()
	for (B,C) in service_backend.items():
		for (D,E) in C.items():F=ServiceKey(account_id=B,region=D,service=api);G=BackendState(F,{STORE_STATE_FILE:E});A.put_backend(G)
	return A
def _service_state_from_backend_state(service_backend,api):
	A=service_backend;from localstack_ext.constants import MOTO_BACKEND_STATE_FILE as D;B=ServiceState()
	if not isinstance(A,dict):
		if isinstance(A,S3Backend):
			if not A.buckets:return B
		A={_A:A}
	for (C,E) in A.items():
		if not C:continue
		for (F,G) in E.items():H=ServiceKey(account_id=C,region=F,service=api);I=BackendState(H,{D:G});B.put_backend(I)
	return B
def _service_state_from_backend(backend,api,memory_management):
	C=memory_management;B=api;A=backend;LOG.debug('Backend for service %s, memory %s, of type %s',B,C,type(A))
	if C=='localstack':return _service_state_from_account_region_bundle(service_backend=A,api=B)
	if C=='moto':return _service_state_from_backend_state(service_backend=A,api=B)

==> localstack-ext-1.4.1.dev20230320115709.tar.gz: localstack-ext-1.4.1.dev20230320115709/localstack_ext/bootstrap/pods/server/states.py <==
_A='global'
import logging,os
from typing import Dict,Optional,Set
from localstack import config
from localstack.aws.accounts import get_aws_account_id
from localstack.services.stores import AccountRegionBundle
from localstack.state.inspect import ReflectionStateLocator,ServiceBackend,ServiceBackendCollectorVisitor
from localstack.utils.aws.aws_stack import get_valid_regions_for_service
from localstack.utils.files import load_file
from localstack.utils.objects import singleton_factory
from localstack.utils.strings import to_bytes
from moto.core import BackendDict
from moto.s3.models import S3Backend
from plugin import PluginManager
from localstack_ext.bootstrap.pods.server.plugins import StateLifecyclePlugin
from localstack_ext.bootstrap.pods.server.visitors import PodStateLoaderVisitor
from localstack_ext.bootstrap.pods.service_state.service_state import ServiceState
from localstack_ext.bootstrap.pods.service_state.service_state_types import AccountRegion,BackendState,ServiceKey
from localstack_ext.constants import STORE_STATE_FILE
LOG=logging.getLogger(__name__)
PLUGIN_NAMESPACE='localstack.state.lifecycle'
class PersistencePluginManager(PluginManager[StateLifecyclePlugin]):
	def __init__(A):super().__init__(PLUGIN_NAMESPACE)
	@staticmethod
	@singleton_factory
	def get():return PersistencePluginManager()
	def get_plugin(B,service):
		A=service
		try:return B.load(A)
		except ValueError as C:LOG.debug('Failed to load persistence plugin for "%s": "%s',A,C)
class ServiceStateManager:
	def __init__(A,service):A.service=service;A.manager=PersistencePluginManager.get();A.manager.load_all()
	def get_service_state(A):
		D=ServiceBackendCollectorVisitor();G=ReflectionStateLocator(service=A.service);G.accept_state_visitor(visitor=D);E=D.collect();B=ServiceState()
		for F in E.keys():B.put_service_state(_service_state_from_backend(backend=E[F],api=A.service,memory_management=F))
		C=A.manager.get_plugin(A.service)
		if C and C.has_assets():H=C.retrieve_assets();B.put_assets(A.service,H)
		return B
	def inject_service_state(A,pod_state,pod_tmp_dir=None):
		from localstack.services.plugins import SERVICE_PLUGINS as E;F=PodStateLoaderVisitor(pod_state=pod_state,service=A.service);G=ReflectionStateLocator(service=A.service);G.accept_state_visitor(visitor=F);B=A.manager.get_plugin(A.service)
		if B:B.inject_assets(pod_asset_directory=pod_tmp_dir)
		C=E.get_service_container(A.service)
		if not C:LOG.debug('No service container for %s',A.service);return
		D=C.service.lifecycle_hook
		if D:
			try:D.on_after_state_load()
			except Exception as H:LOG.debug('An error occurred while executing the after inject hook for %s: %s',A.service,H)
	def active_service_regions(B):
		A=B._active_service_regions()
		if B.service=='stepfunctions':
			C=os.path.join(config.dirs.data,B.service,'backend_state')
			if os.path.exists(C):
				try:D=load_file(C,mode='rb');A=[A for A in A if to_bytes(A.region)in D]
				except Exception:pass
			return set(A)
		return A
	def _active_service_regions(D):
		def B(reg):return AccountRegion(account_id=get_aws_account_id(),region=reg)
		def G(account_id):
			A=account_id
			if A=='123456789012':return get_aws_account_id()
			return A
		E=ServiceBackendCollectorVisitor();H=ReflectionStateLocator(service=D.service);H.accept_state_visitor(visitor=E);F=E.collect()
		if not F:I=get_valid_regions_for_service(D.service);return set([B(A)for A in I])
		C=set()
		for (M,A) in F.items():
			if isinstance(A,(BackendDict,AccountRegionBundle)):
				for (J,K) in A.items():
					for L in K.keys():C.add(AccountRegion(account_id=G(J),region=L))
			elif isinstance(A,dict):C.update(set([B(A)for A in A.keys()]))
			else:return{B(_A)}
		return C
def _service_state_from_account_region_bundle(service_backend,api):
	A=ServiceState()
	for (B,C) in service_backend.items():
		for (D,E) in C.items():F=ServiceKey(account_id=B,region=D,service=api);G=BackendState(F,{STORE_STATE_FILE:E});A.put_backend(G)
	return A
def _service_state_from_backend_state(service_backend,api):
	A=service_backend;from localstack_ext.constants import MOTO_BACKEND_STATE_FILE as D;B=ServiceState()
	if not isinstance(A,dict):
		if isinstance(A,S3Backend):
			if not A.buckets:return B
		A={_A:A}
	for (C,E) in A.items():
		if not C:continue
		for (F,G) in E.items():H=ServiceKey(account_id=C,region=F,service=api);I=BackendState(H,{D:G});B.put_backend(I)
	return B
def _service_state_from_backend(backend,api,memory_management):
	C=memory_management;B=api;A=backend;LOG.debug('Backend for service %s, memory %s, of type %s',B,C,type(A))
	if C=='localstack':return _service_state_from_account_region_bundle(service_backend=A,api=B)
	if C=='moto':return _service_state_from_backend_state(service_backend=A,api=B)
